/*
  This program tests your implentations of StaticAnalysis.java
  and AST2infix.java. When your implentations are correct,
  the output from this program should look exactly like
  the contents of the file output.txt.

  You should consider creating your own test expressions to make
  sure your implementations are correct.
*/

public class Tests
{
   public static void main(String[] args)
   {
      String[] exprs =
      {
         "( prog " +
           "( define x ( + x 1 ) ) " +  // can't have a recursive declaration
           "( set u ( + 0 false ) ) )",

         "( prog " +
           "( define z 10 ) " +
           "( define f ( lambda x " +
              "( begin " +
                "( if true z ( while false ( + u v ) ) ) " +
                "( apply f x ) ) ) ) " +
           "( apply f z ) )",

         "( prog " +
           "( begin " +
             "( var i1 5 ) " +
             "( while ( > i2 0 ) " +
               "( begin " +
                 "( print i1 ) " +
                 "( var i3 ( - i3 1 ) ) ) ) ) )",

         "( prog " +
           "( begin " +
            "( var n 1 ) " +
            "( var sum 0 ) " +
            "( while ( < n 5 ) " +
              "( begin " +
                "( set sum ( + sum n ) ) " +
                "( set n ( + n 1 ) ) " +
                "( print n ) " +
                "( print sum ) ) ) " +
            "sum ) )",

        "( prog " +
          "( define y 3 ) " +
          "( define f ( lambda x ( + x y ) ) ) " +
          "( define g ( lambda u ( * u y ) ) ) " +
          "( define h ( lambda v ( && v u ) ) ) " +
          "( apply  g f ) )", // function parameters not allowed

        "( prog ( var x ( apply ( lambda x ( * x x ) ) x ) ) )",

        "( prog ( var y ( apply ( lambda x ( * x y ) ) 0 ) ) )",

        "( prog ( define y 10 ) ( var y ( apply ( lambda x ( * x y ) ) 0 ) ) )",

        "( prog " +
           "( define cAdd ( lambda a ( begin ( var r ( lambda b ( + b a ) ) ) r ) ) ) " +
           "( define cAdd2 ( apply cAdd 2 ) ) " +  // function return values not allowed
           "( apply cAdd2 3 ) )",

         "( prog " +
           "( define x 20 ) " +
           "( begin " +
             "( var x 10 ) " +
             "( apply ( lambda z ( + z x ) ) 5 ) ) )"
      };

      for (int i = 0; i < exprs.length; i++)
      {
         Tree ast = BuildTree.buildTree( exprs[i] );

         System.out.println( PrettyPrinter.prettyPrint( ast ) + "\n" );
         System.out.println( AST2infix.ast2infix( ast ) + "\n" );

         StaticAnalysis.analyze( ast );

         System.out.println("=============================================");
      }
   }//main

}//Tests